/*
 *  glue.c
 *  Derived from q4linux
 *
 *  Converts QuickTime movie files to AVI format
 *
 *  Modified by Benoit Joossen (July 2005).
 *  Company : Aero Quartet
 *
 *  You may copy, modify and redistribute this source file under
 *  the terms of the GNU Public License
 *
 */

#include "qtprivate.h"
#include "glue.h"
#include "funcprotos.h"

float audioSamplesAccumulator = 0.0;
int audioSamples = 0;
int videoSamples = 0;
int audioFrames = 0;
int audioSync = 0; // First frame of audio
int sampleSize = 0;
int sampleRate = 0;
int audioFactor = 0;
float audioLever = 0;
int mediaSampleSize = 0;
int mediaRefs = 0;
Boolean isAudioCompressed = false;
Boolean audioSyncSupported = true;
int packetSize = 0;
Boolean isSpecialADPCM = false;
Boolean reverse16 = false;
Boolean copySampleDescription = false;
SoundDescriptionHandle sdh;
unsigned char* readBuffer[100];
off_t bufferPos[100];
FILE* refFiles[100];
float globalFPS;

// For AVI
int quicktime_avg_chunk_samples(quicktime_t *file, quicktime_trak_t *trak)
{
	return file->moov.trak[1]->mdia.minf.stbl.stsd.table[0].samples_per_packet; // 16 Dec 2007
	return 1;
	/*
	int i, chunk = trak->mdia.minf.stbl.stco.total_entries - 1;
	long total_samples;

	if(chunk >= 0)
	{
		long GetMediaSampleCount (Media    theMedia );
		OSErr GetMediaPreferredChunkSize (Media    theMedia, long     *maxChunkSize );

		total_samples = quicktime_sample_of_chunk(trak, chunk);
		return total_samples / (chunk + 1);
	}
	else
	{
		total_samples = quicktime_track_samples(file, trak);
		return total_samples;
	} */
}

long quicktime_track_samples(quicktime_t *file, quicktime_trak_t *trak)
{
	if (trak->mdia.minf.is_video) return videoSamples;
	else return audioSamples;
printf("file->rd %d file->wr %d\n", file->rd, file->wr);
	if(file->wr)
	{
// get the sample count when creating a new file 
 		quicktime_stsc_table_t *table = trak->mdia.minf.stbl.stsc.table;
		long total_entries = trak->mdia.minf.stbl.stsc.total_entries;
		long chunk = trak->mdia.minf.stbl.stco.total_entries;
		long sample;

		if(chunk)
		{
			sample = quicktime_sample_of_chunk(trak, chunk);
			//sample += table[total_entries - 1].samples;
		}
		else 
			sample = 0;

		return sample;
	}
	else
	{
// get the sample count when reading only 
		quicktime_stts_t *stts = &(trak->mdia.minf.stbl.stts);
		int i;
		int64_t total = 0;

		for(i = 0; i < stts->total_entries; i++)
		{
			total += stts->table[i].sample_count *
				stts->table[i].sample_duration;
		}

// Convert to samples 
		if(trak->mdia.minf.is_audio)
			return total;
		else
		if(trak->mdia.minf.is_video)
			return total /
				stts->table[0].sample_duration;
		return total;
	}
}

long quicktime_sample_of_chunk(quicktime_trak_t *trak, long chunk)
{
	quicktime_stsc_table_t *table = trak->mdia.minf.stbl.stsc.table;
	long total_entries = trak->mdia.minf.stbl.stsc.total_entries;
	long chunk1entry, chunk2entry;
	long chunk1, chunk2, chunks, total = 0;

	for(chunk1entry = total_entries - 1, chunk2entry = total_entries; 
		chunk1entry >= 0; 
		chunk1entry--, chunk2entry--)
	{
		chunk1 = table[chunk1entry].chunk;

		if(chunk > chunk1)
		{
			if(chunk2entry < total_entries)
			{
				chunk2 = table[chunk2entry].chunk;

				if(chunk < chunk2) chunk2 = chunk;
			}
			else
				chunk2 = chunk;

			chunks = chunk2 - chunk1;

			total += chunks * table[chunk1entry].samples;
		}
	}

	return total;
}

double quicktime_frame_rate(quicktime_t *file, int track)
{
	if(file->total_vtracks > track)
		return (float)file->vtracks[track].track->mdia.mdhd.time_scale / 
			file->vtracks[track].track->mdia.minf.stbl.stts.table[0].sample_duration;

	return 0;
}

void quicktime_write_chunk_header(quicktime_t *file, 
	quicktime_trak_t *trak, 
	quicktime_atom_t *chunk)
{
	if(file->use_avi)
	{
/* Get tag from first riff strl */
		quicktime_riff_t *first_riff = file->riff[0];
		quicktime_hdrl_t *hdrl = &first_riff->hdrl;
		quicktime_strl_t *strl = hdrl->strl[trak->tkhd.track_id - 1];
		char *tag = strl->tag;
		/*if (strl->tag == 0) {
			printf("Wrong pointer\n");
			return;
		}
		int tagInt = *(int *)strl->tag;
		if ((tagInt != '01wb') && (tagInt != '00dc')) {
			printf("Unknown tag\n");
			return;
		}*/
/* Create new RIFF object at 1 Gig mark */
		quicktime_riff_t *riff = file->riff[file->total_riffs - 1];
		if(quicktime_position(file) - riff->atom.start > 0x40000000)
		{
			quicktime_finalize_riff(file, riff);
			quicktime_init_riff(file);
		}

		

/* Write AVI header */
		quicktime_atom_write_header(file, chunk, tag);
	}
	else
	{
		chunk->start = quicktime_position(file);
	}
}

void quicktime_write_chunk_footer(quicktime_t *file, 
	quicktime_trak_t *trak,
	int current_chunk,
	quicktime_atom_t *chunk, 
	int samples)
{
	int64_t offset = chunk->start;
	int sample_size = quicktime_position(file) - offset;


// Write AVI footer
	if(file->use_avi)
	{
		quicktime_atom_write_footer(file, chunk);

// Save original index entry for first RIFF only
		if(file->total_riffs < 2)
		{
			quicktime_update_idx1table(file, 
				trak, 
				offset, 
				sample_size);
		}

// Save partial index entry
		quicktime_update_ixtable(file, 
			trak, 
			offset, 
			sample_size);
	}

	if(offset + sample_size > file->mdat.atom.size)
		file->mdat.atom.size = offset + sample_size;

	//if(current_chunk > trak->mdia.minf.stbl.stco.total_entries) trak->mdia.minf.stbl.stco.total_entries = current_chunk;
	if (0) { //trak->mdia.minf.is_audio) {
	quicktime_update_stco(&(trak->mdia.minf.stbl.stco), 
		current_chunk, 
		offset);
	}
/*
	if(trak->mdia.minf.is_video)
		quicktime_update_stsz(&(trak->mdia.minf.stbl.stsz), 
		current_chunk - 1, 
		sample_size);

	quicktime_update_stsc(&(trak->mdia.minf.stbl.stsc), 
		current_chunk, 
		samples); */
}

void quicktime_insert_keyframe(quicktime_t *file, long frame, int track)
{
	quicktime_trak_t *trak = file->vtracks[track].track;
	quicktime_stss_t *stss = &trak->mdia.minf.stbl.stss;
	int i;

// Set keyframe flag in idx1 table.
// Only possible in the first RIFF.  After that, there's no keyframe support.
	if ((file->use_avi) && (file->total_riffs == 1)) // BJ 07 Sep 2005 
		quicktime_set_idx1_keyframe(file, 
			trak,
			frame, frame);
// BJ 11 Sep 2005
// Assuming we call this just after inserting the video frame,
// We can modify the latest record in ix00
	

// Offset 1
	frame++;


// Get the keyframe greater or equal to new frame
	for(i = 0; i < stss->total_entries; i++)
	{
		if(stss->table[i].sample >= frame) break;
	}

// Expand table
	if(stss->entries_allocated <= stss->total_entries)
	{
		stss->entries_allocated *= 2;
		if (stss->entries_allocated == 0) stss->entries_allocated = 1; // 2 Dec 2006
		stss->table = realloc(stss->table, sizeof(quicktime_stss_table_t) * stss->entries_allocated);
	}

// Insert before existing frame
	if(i < stss->total_entries)
	{
		if(stss->table[i].sample > frame)
		{
			int j, k;
			for(j = stss->total_entries, k = stss->total_entries - 1;
				k >= i;
				j--, k--)
			{
				stss->table[j] = stss->table[k];
			}
			stss->table[i].sample = frame;
		}
	}
	else
// Insert after last frame
		stss->table[i].sample = frame;

	stss->total_entries++;
}

